@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 #
 # This file is part of the python-chess library.
-# Copyright (C) 2012-2015 Niklas Fiekas <niklas.fiekas@tu-clausthal.de>
+# Copyright (C) 2012-2016 Niklas Fiekas <niklas.fiekas@backscattering.de>
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -83,7 +83,8 @@ MOVETEXT_REGEX = re.compile(r"""
     |(\))
     |(\*|1-0|0-1|1/2-1/2)
     |(
-        [NBKRQ]?[a-h]?[1-8]?[\-x]?[a-h][1-8](?:=?[nbrqNBRQ])?
+        [NBKRQ]?[a-h]?[1-8]?[\-x]?[a-h][1-8](?:=?[nbrqkNBRQK])?
+        |[PNBRQK]?@[a-h][1-8]
         |--
         |O-O(?:-O)?
         |0-0(?:-0)?
@@ -267,7 +268,7 @@ class GameNode(object):
 
         # Merge comment and NAGs.
         if node.comment:
-            node.comment +=  " " + comment
+            node.comment += " " + comment
         else:
             node.comment = comment
 
@@ -388,13 +389,18 @@ class Game(GameNode):
         Unless the `SetUp` and `FEN` header tags are set this is the default
         starting position.
         """
-        if "FEN" in self.headers and self.headers.get("SetUp", "1") == "1":
-            chess960 = self.headers.get("Variant") == "Chess960"
-            board = chess.Board(self.headers["FEN"], chess960=chess960)
-            board.chess960 = board.chess960 or board.has_chess960_castling_rights()
-            return board
+        chess960 = self.headers.get("Variant", "").lower() == "chess960"
+
+        if chess960 or "Variant" not in self.headers:
+            VariantBoard = chess.Board
         else:
-            return chess.Board()
+            from chess.variant import find_variant
+            VariantBoard = find_variant(self.headers["Variant"])
+
+        fen = self.headers.get("FEN") if self.headers.get("SetUp", "1") == "1" else None
+        board = VariantBoard(fen or VariantBoard.starting_fen, chess960)
+        board.chess960 = board.chess960 or board.has_chess960_castling_rights()
+        return board
 
     def setup(self, board):
         """
@@ -408,15 +414,18 @@ class Game(GameNode):
             board.chess960 = board.has_chess960_castling_rights()
             fen = board.fen()
 
-        if fen == chess.STARTING_FEN:
+        if fen == type(board).starting_fen:
             self.headers.pop("SetUp", None)
             self.headers.pop("FEN", None)
         else:
             self.headers["SetUp"] = "1"
             self.headers["FEN"] = fen
 
-        if board.chess960:
+        if type(board).aliases[0] == "Standard" and board.chess960:
             self.headers["Variant"] = "Chess960"
+        elif type(board).aliases[0] != "Standard":
+            self.headers["Variant"] = type(board).aliases[0]
+            self.headers["FEN"] = board.fen()
         else:
             self.headers.pop("Variant", None)
 
@@ -478,10 +487,6 @@ class BaseVisitor(object):
         """Called at the start of a game."""
         pass
 
-    def end_game(self):
-        """Called at the end of a game."""
-        pass
-
     def begin_headers(self):
         """Called at the start of the game headers."""
         pass
@@ -494,6 +499,23 @@ class BaseVisitor(object):
         """Called at the end of the game headers."""
         pass
 
+    def visit_move(self, board, move):
+        """
+        Called for each move.
+
+        *board* is the board state before the move. The board state must be
+        restored before the traversal continues.
+        """
+        pass
+
+    def visit_comment(self, comment):
+        """Called for each comment."""
+        pass
+
+    def visit_nag(self, nag):
+        """Called for each NAG."""
+        pass
+
     def begin_variation(self):
         """
         Called at the start of a new variation. It is not called for the
@@ -505,35 +527,22 @@ class BaseVisitor(object):
         """Concludes a variation."""
         pass
 
-    def visit_comment(self, comment):
-        """Called for each comment."""
-        pass
-
-    def visit_nag(self, nag):
-        """Called for each NAG."""
-        pass
-
-    def visit_move(self, board, move):
-        """
-        Called for each move.
-
-        *board* is the board state before the move. The board state must be
-        restored before the traversal continues.
-        """
-        pass
-
     def visit_result(self, result):
         """Called at the end of the game with the *Result*-header."""
         pass
 
+    def end_game(self):
+        """Called at the end of a game."""
+        pass
+
+    def result(self):
+        """Called to get the result of the visitor. Defaults to ``True``."""
+        return True
+
     def handle_error(self, error):
         """Called for errors encountered. Defaults to raising an exception."""
         raise error
 
-    def result(self):
-        """Called to get the result of the visitor. Defaults to ``None``."""
-        return None
-
 
 class GameModelCreator(BaseVisitor):
     """
@@ -542,15 +551,11 @@ class GameModelCreator(BaseVisitor):
 
     def __init__(self):
         self.game = Game()
-        self.found_game = False
 
         self.variation_stack = collections.deque([self.game])
         self.starting_comment = ""
         self.in_variation = False
 
-    def begin_game(self):
-        self.found_game = True
-
     def visit_header(self, tagname, tagvalue):
         self.game.headers[tagname] = tagvalue
 
@@ -587,15 +592,18 @@ class GameModelCreator(BaseVisitor):
         self.in_variation = True
 
     def handle_error(self, error):
+        """
+        Populates :data:`chess.pgn.Game.errors` with encountered errors and
+        logs them.
+        """
         LOGGER.exception("error during pgn parsing")
         self.game.errors.append(error)
 
     def result(self):
         """
-        Returns a :class:`~chess.pgn.Game()` or ``None`` if no game was
-        encountered.
+        Returns the visited :class:`~chess.pgn.Game()`.
         """
-        return self.game if self.found_game else None
+        return self.game
 
 
 class StringExporter(BaseVisitor):
@@ -800,8 +808,7 @@ def read_game(handle, Visitor=GameModelCreator):
         if not found_game:
             visitor.begin_game()
             visitor.begin_headers()
-
-        found_game = True
+            found_game = True
 
         # Read header tags.
         tag_match = TAG_REGEX.match(line)
@@ -835,8 +842,9 @@ def read_game(handle, Visitor=GameModelCreator):
         if not line.strip() and found_content:
             if found_game:
                 visitor.end_game()
-
-            return visitor.result()
+                return visitor.result()
+            else:
+                return
 
         for match in MOVETEXT_REGEX.finditer(line):
             token = match.group(0)
@@ -931,8 +939,7 @@ def read_game(handle, Visitor=GameModelCreator):
 
     if found_game:
         visitor.end_game()
-
-    return visitor.result()
+        return visitor.result()
 
 
 def scan_headers(handle):
