@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 #
 # This file is part of the python-chess library.
-# Copyright (C) 2012-2015 Niklas Fiekas <niklas.fiekas@tu-clausthal.de>
+# Copyright (C) 2012-2016 Niklas Fiekas <niklas.fiekas@backscattering.de>
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -21,7 +21,6 @@ import struct
 import os
 import mmap
 import random
-import itertools
 
 try:
     import backport_collections as collections
@@ -57,7 +56,10 @@ class Entry(collections.namedtuple("Entry", ["key", "raw_move", "weight", "learn
                 elif to_square == chess.A8:
                     return chess.Move(chess.E8, chess.C8)
 
-        return chess.Move(from_square, to_square, promotion)
+        if promotion and from_square == to_square:
+            return chess.Move(from_square, to_square, drop=promotion)
+        else:
+            return chess.Move(from_square, to_square, promotion)
 
 
 class MemoryMappedReader(object):
@@ -176,7 +178,7 @@ class MemoryMappedReader(object):
         to delete entries from an opening book without compacting it. Pass
         *minimum_weight* ``0`` to select all entries.
 
-        Raises :exc:`IndexError` if no entries are found.
+        :raises: :exc:`IndexError` if no entries are found.
         """
         try:
             return max(self.find_all(board, minimum_weight, exclude_moves), key=lambda entry: entry.weight)
@@ -187,7 +189,7 @@ class MemoryMappedReader(object):
         """
         Uniformly selects a random entry for the given position.
 
-        Raises :exc:`IndexError` if no entries are found.
+        :raises: :exc:`IndexError` if no entries are found.
         """
         chosen_entry = None
 
@@ -205,7 +207,7 @@ class MemoryMappedReader(object):
         Selects a random entry for the given position, distributed by the
         weights of the entries.
 
-        Raises :exc:`IndexError` if no entries are found.
+        :raises: :exc:`IndexError` if no entries are found.
         """
         total_weights = sum(entry.weight for entry in self.find_all(board, exclude_moves=exclude_moves))
         if not total_weights:
