@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 #
 # This file is part of the python-chess library.
-# Copyright (C) 2012-2015 Niklas Fiekas <niklas.fiekas@tu-clausthal.de>
+# Copyright (C) 2012-2016 Niklas Fiekas <niklas.fiekas@backscattering.de>
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -505,6 +505,7 @@ class Engine(object):
 
         self.board = chess.Board()
         self.uci_chess960 = None
+        self.uci_variant = None
 
         self.name = None
         self.author = None
@@ -588,9 +589,11 @@ class Engine(object):
             return
 
     def _uciok(self):
-        # Set UCI_Chess960 default value.
+        # Set UCI_Chess960 and UCI_Variant default value.
         if self.uci_chess960 is None and "UCI_Chess960" in self.options:
             self.uci_chess960 = self.options["UCI_Chess960"].default
+        if self.uci_variant is None and "UCI_Variant" in self.options:
+            self.uci_variant = self.options["UCI_Variant"].default
 
         self.uciok.set()
 
@@ -620,7 +623,7 @@ class Engine(object):
                 self.ponder = chess.Move.from_uci(tokens[2])
                 if self.ponder.from_square in [chess.E1, chess.E8] and self.ponder.to_square in [chess.C1, chess.C8, chess.G1, chess.G8]:
                     # Make a copy of the board to avoid race conditions.
-                    board = self.board.copy()
+                    board = self.board.copy(stack=False)
                     board.push(self.bestmove)
                     self.ponder = board.parse_uci(tokens[2])
             except ValueError:
@@ -633,11 +636,9 @@ class Engine(object):
             info_handler.on_bestmove(self.bestmove, self.ponder)
 
     def _copyprotection(self, arg):
-        # TODO: Implement copyprotection
         LOGGER.error("engine copyprotection not supported")
 
     def _registration(self, arg):
-        # TODO: Implement registration
         LOGGER.error("engine registration not supported")
 
     def _info(self, arg):
@@ -746,7 +747,7 @@ class Engine(object):
                     pv = []
 
                 if current_parameter in ["refutation", "pv", "currline"]:
-                    board = self.board.copy()
+                    board = self.board.copy(stack=False)
             elif current_parameter == "depth":
                 handle_integer_token(token, lambda handler, val: handler.depth(val))
             elif current_parameter == "seldepth":
@@ -975,6 +976,8 @@ class Engine(object):
         for name, value in options.items():
             if name.lower() == "uci_chess960":
                 self.uci_chess960 = value
+            if name.lower() == "uci_variant":
+                self.uci_variant = value.lower()
 
             builder = []
             builder.append("setoption name")
@@ -1005,8 +1008,6 @@ class Engine(object):
 
         return self._queue_command(command, async_callback)
 
-    # TODO: Implement register command
-
     def ucinewgame(self, async_callback=None):
         """
         Tell the engine that the next search will be from a different game.
@@ -1053,8 +1054,12 @@ class Engine(object):
         :raises: :exc:`~chess.uci.EngineStateException` if the engine is still
             calculating.
         """
-        # Work on a local copy.
-        board = board.copy()
+        # Check UCI_Variant
+        uci_variant = type(board).uci_variant
+        if uci_variant == "chess" and self.uci_variant is None:
+            pass
+        elif uci_variant != self.uci_variant:
+            LOGGER.error("current UCI_Variant (%s) does not match position (%s)", self.uci_variant, uci_variant)
 
         # Raise if this is called while the engine is still calculating.
         with self.state_changed:
@@ -1064,32 +1069,39 @@ class Engine(object):
         builder = []
         builder.append("position")
 
-        # Take back moves to obtain the first FEN we know. Later giving the
-        # moves explicitly allows for transposition detection.
+        # Take back moves to obtain the FEN at the latest pawn move or
+        # capture. Later giving the moves explicitly allows for transposition
+        # detection.
         switchyard = collections.deque()
         while board.move_stack:
-            switchyard.append(board.pop())
+            move = board.pop()
+            switchyard.append(move)
+
+            if board.is_zeroing(move):
+                break
 
         # Validate castling rights.
         if not self.uci_chess960 and board.chess960:
             if board.has_chess960_castling_rights():
                 LOGGER.error("not in UCI_Chess960 mode but position has non-standard castling rights")
 
-                # Just send the final FEN without transpositions in hops
+                # Just send the final FEN without transpositions in hopes
                 # that this will work.
                 while switchyard:
                     board.push(switchyard.pop())
 
         # Send startposition.
-        if board.fen() == chess.STARTING_FEN:
+        if uci_variant == "chess" and board.fen() == chess.STARTING_FEN:
             builder.append("startpos")
         else:
             builder.append("fen")
 
+            promoted = uci_variant in ["giveaway", "suicide"] and board.has_chess960_castling_rights()
+
             if self.uci_chess960:
-                builder.append(board.shredder_fen())
+                builder.append(board.shredder_fen(promoted=promoted))
             else:
-                builder.append(board.fen())
+                builder.append(board.fen(promoted=promoted))
 
         # Send moves.
         if switchyard:
@@ -1100,18 +1112,14 @@ class Engine(object):
                 builder.append(board.uci(move, chess960=self.uci_chess960))
                 board.push(move)
 
-        self.board = board
+        self.board = board.copy(stack=False)
 
         def command():
             with self.semaphore:
-                with self.readyok_received:
-                    self.send_line(" ".join(builder))
+                self.send_line(" ".join(builder))
 
-                    self.send_line("isready")
-                    self.readyok_received.wait()
-
-                    if self.terminated.is_set():
-                        raise EngineTerminatedException()
+                if self.terminated.is_set():
+                    raise EngineTerminatedException()
 
         return self._queue_command(command, async_callback)
 
@@ -1240,6 +1248,9 @@ class Engine(object):
         def command():
             with self.semaphore:
                 with self.state_changed:
+                    if not self.idle:
+                        self.search_started.wait()
+
                     backoff = 0.5
                     while not self.bestmove_received.is_set() and not self.terminated.is_set():
                         if self.idle:
